==> 0.1.1.tar.gz: lib/src/eager_query_impl.dart <==
// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _EagerQueryImpl<E> implements Query<E> {
  List<E> list;

  _EagerQueryImpl(Iterable<E> iterable) : list = new List.from(iterable);

  Iterator<E> iterator() => list.iterator();

  Query<E> append(Iterable<E> other) {
    list.addAll(new List<E>.from(other));
    return this;
  }

  // I hate it, but it's O(N**2) now...
  Query<E> unique() {
    bool contains(List<E> list, E element) => list.some( (e) => e == element );

    List<E> result = new List<E>();
    for (E e in list) {
      if (!contains(result, e)) {
        result.add(e);
      }
    }

    list = result;
    return this;
  }

  Query<E> filter(predicate) {
    list = list.filter(positivePredicate(predicate));
    return this;
  }

  Query<E> reject(predicate) {
    list = list.filter(negativePredicate(predicate));
    return this;
  }

  Query<E> sort([int compare(E a, E b) = null]) {
    if (compare == null) {
      compare = (E a, E b) => a.compareTo(b);
    }

    list.sort(compare);
    return this;
  }

  Query<E> sortBy(extractCompareKey(E e)) {
    Comparable comparableFromKey(key) {
      if (key is Comparable) {
        return key;
      } else if (key is List<Comparable>) {
        return new _CompoundComparable(key);
      } else {
        throw new ExpectException("Compare key must be either Comparable or List<Comparable>");
      }
    }

    list.sort(int (E a, E b) {
      var aCmp = comparableFromKey(extractCompareKey(a));
      var bCmp = comparableFromKey(extractCompareKey(b));
      return aCmp.compareTo(bCmp);
    });
    return this;
  }

  Query<E> reverse() {
    List<E> result = new List<E>();
    for (int i = list.length - 1; i >= 0; i--) {
      result.add(list[i]);
    }

    list = result;
    return this;
  }

  Query/*<F>*/ map(/*F*/ transform(E e)) {
    List/*<F>*/ result = new List/*<F>*/();
    for (final e in list) {
      result.add(transform(e));
    }

    return new _EagerQueryImpl/*<F>*/(result);
  }

  Query<E> skip(int count) {
    if (count >= list.length) {
      list = new List<E>();
    } else {
      list = list.getRange(count, list.length - count);
    }
    return this;
  }

  Query<E> take(int count) {
    if (count > list.length) {
      count = list.length;
    }

    list = list.getRange(0, count);
    return this;
  }

  int _countFirst(predicate) {
    var p = positivePredicate(predicate);

    int count = 0;
    for (final e in list) {
      if (p(e)) {
        count += 1;
      } else {
        break;
      }
    }
    return count;
  }

  Query<E> skipWhile(predicate) => skip(_countFirst(predicate));

  Query<E> takeWhile(predicate) => take(_countFirst(predicate));

  bool isEmpty() => list.isEmpty();

  bool some(predicate) => list.some(positivePredicate(predicate));

  bool every(predicate) => list.every(positivePredicate(predicate));

  bool contains(E e) => some( (item) => e == item );

  E findFirst(predicate) {
    final p = positivePredicate(predicate);
    for (final e in list) {
      if (p(e)) {
        return e;
      }
    }

    return null; // TODO what else?
  }

  /*F*/ fold(/*F*/ seed, /*F*/ aggregate(/*F*/ Object accumulator, E e)) {
    /*F*/ var acc = seed;
    for (final e in list) {
      acc = aggregate(acc, e);
    }

    return acc;
  }

  /*F*/ foldRight(/*F*/ seed, /*F*/ aggregate(/*F*/ Object accumulator, E e)) {
    /*F*/ var acc = seed;
    for (int i = list.length - 1; i >= 0; i-=1) {
      var e = list[i];
      acc = aggregate(acc, e);
    }

    return acc;
  }

  E reduce(E aggregate(E accumulator, E e)) {
    if (list.isEmpty()) {
      return null;
    }

    var acc = list[0];
    for (int i = 1; i < list.length; i+=1) {
      var e = list[i];
      acc = aggregate(acc, e);
    }
    return acc;
  }

  E reduceRight(E aggregate(E accumulator, E e)) {
    if (list.isEmpty()) {
      return null;
    }

    var acc = list[list.length - 1];
    for (int i = list.length - 2; i >= 0; i-=1) {
      var e = list[i];
      acc = aggregate(acc, e);
    }
    return acc;
  }

  E max() {
    if (list.isEmpty()) {
      return null;
    }

    E currentMax = list[0];
    for (final e in list) {
      if (e > currentMax) {
        currentMax = e;
      }
    }

    return currentMax;
  }

  E min() {
    if (list.isEmpty()) {
      return null;
    }

    E currentMin = list[0];
    for (final e in list) {
      if (e < currentMin) {
        currentMin = e;
      }
    }

    return currentMin;
  }

  E sum() {
    if (list.isEmpty()) {
      return null;
    }

    E acc = list[0];
    if (list.length > 1) {
      for (int i = 1; i < list.length; i++) {
        acc += list[i];
      }
    }

    return acc;
  }

  String asString(String separator, [bool skipNulls = true]) {
    List<String> strings = new List<String>();
    for (final o in list) {
      if (skipNulls && o === null) {
        continue;
      }
      strings.add(o.toString());
    }

    return Strings.join(strings, separator);
  }

  E operator [](int index) => index >= 0 ? list[index] : list[list.length + index];
}

==> 0.1.2.tar.gz: lib/src/eager_query_impl.dart <==
// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of deequery;

class _EagerQueryImpl<E> implements Query<E> {
  List<E> _list;

  _EagerQueryImpl(Iterable<E> iterable) : _list = new List.from(iterable);

  Iterator<E> iterator() => _list.iterator();

  Query<E> append(Iterable<E> other) {
    _list.addAll(new List<E>.from(other));
    return this;
  }

  // I hate it, but it's O(N**2) now...
  Query<E> unique() {
    bool contains(List<E> list, E element) => list.some( (e) => e == element );

    List<E> result = new List<E>();
    for (E e in _list) {
      if (!contains(result, e)) {
        result.add(e);
      }
    }

    _list = result;
    return this;
  }

  Query<E> filter(predicate) {
    _list = _list.filter(positivePredicate(predicate));
    return this;
  }

  Query<E> reject(predicate) {
    _list = _list.filter(negativePredicate(predicate));
    return this;
  }

  Query<E> sort([int compare(E a, E b) = null]) {
    if (compare == null) {
      compare = (E a, E b) => a.compareTo(b);
    }

    _list.sort(compare);
    return this;
  }

  Query<E> sortBy(extractCompareKey(E e)) {
    Comparable comparableFromKey(key) {
      if (key is Comparable) {
        return key;
      } else if (key is List<Comparable>) {
        return new _CompoundComparable(key);
      } else {
        throw new ExpectException("Compare key must be either Comparable or List<Comparable>");
      }
    }

    _list.sort(int (E a, E b) {
      var aCmp = comparableFromKey(extractCompareKey(a));
      var bCmp = comparableFromKey(extractCompareKey(b));
      return aCmp.compareTo(bCmp);
    });
    return this;
  }

  Query<E> reverse() {
    List<E> result = new List<E>();
    for (int i = _list.length - 1; i >= 0; i--) {
      result.add(_list[i]);
    }

    _list = result;
    return this;
  }

  Query/*<F>*/ map(/*F*/ transform(E e)) {
    List/*<F>*/ result = new List/*<F>*/();
    for (final e in _list) {
      result.add(transform(e));
    }

    return new _EagerQueryImpl/*<F>*/(result);
  }

  Query<E> skip(int count) {
    if (count >= _list.length) {
      _list = new List<E>();
    } else {
      _list = _list.getRange(count, _list.length - count);
    }
    return this;
  }

  Query<E> take(int count) {
    if (count > _list.length) {
      count = _list.length;
    }

    _list = _list.getRange(0, count);
    return this;
  }

  int _countFirst(predicate) {
    var p = positivePredicate(predicate);

    int count = 0;
    for (final e in _list) {
      if (p(e)) {
        count += 1;
      } else {
        break;
      }
    }
    return count;
  }

  Query<E> skipWhile(predicate) => skip(_countFirst(predicate));

  Query<E> takeWhile(predicate) => take(_countFirst(predicate));

  bool isEmpty() => _list.isEmpty;

  bool some(predicate) => _list.some(positivePredicate(predicate));

  bool every(predicate) => _list.every(positivePredicate(predicate));

  bool contains(E e) => some( (item) => e == item );

  E findFirst(predicate) {
    final p = positivePredicate(predicate);
    for (final e in _list) {
      if (p(e)) {
        return e;
      }
    }

    return null; // TODO what else?
  }

  /*F*/ fold(/*F*/ seed, /*F*/ aggregate(/*F*/ Object accumulator, E e)) {
    /*F*/ var acc = seed;
    for (final e in _list) {
      acc = aggregate(acc, e);
    }

    return acc;
  }

  /*F*/ foldRight(/*F*/ seed, /*F*/ aggregate(/*F*/ Object accumulator, E e)) {
    /*F*/ var acc = seed;
    for (int i = _list.length - 1; i >= 0; i-=1) {
      var e = _list[i];
      acc = aggregate(acc, e);
    }

    return acc;
  }

  E reduce(E aggregate(E accumulator, E e)) {
    if (_list.isEmpty) {
      return null;
    }

    var acc = _list[0];
    for (int i = 1; i < _list.length; i+=1) {
      var e = _list[i];
      acc = aggregate(acc, e);
    }
    return acc;
  }

  E reduceRight(E aggregate(E accumulator, E e)) {
    if (_list.isEmpty) {
      return null;
    }

    var acc = _list[_list.length - 1];
    for (int i = _list.length - 2; i >= 0; i-=1) {
      var e = _list[i];
      acc = aggregate(acc, e);
    }
    return acc;
  }

  E max() {
    if (_list.isEmpty) {
      return null;
    }

    E currentMax = _list[0];
    for (final e in _list) {
      if (e > currentMax) {
        currentMax = e;
      }
    }

    return currentMax;
  }

  E min() {
    if (_list.isEmpty) {
      return null;
    }

    E currentMin = _list[0];
    for (final e in _list) {
      if (e < currentMin) {
        currentMin = e;
      }
    }

    return currentMin;
  }

  E sum() {
    if (_list.isEmpty) {
      return null;
    }

    E acc = _list[0];
    if (_list.length > 1) {
      for (int i = 1; i < _list.length; i++) {
        acc += _list[i];
      }
    }

    return acc;
  }

  String asString(String separator, {bool skipNulls: true}) {
    List<String> strings = new List<String>();
    for (final o in _list) {
      if (skipNulls && o === null) {
        continue;
      }
      strings.add(o.toString());
    }

    return Strings.join(strings, separator);
  }

  E operator [](int index) => index >= 0 ? _list[index] : _list[_list.length + index];
}
